/**
 * ASCIIMath Plugin for TinyMCE editor
 *   port of ASCIIMath plugin for HTMLArea written by
 *   David Lippman & Peter Jipsen
 *
 * @author David Lippman
 * @copyright Copyright � 2008 David Lippman.
 *
 * Plugin format based on code that is:
 * @copyright Copyright � 2004-2008, Moxiecode Systems AB, All rights reserved.
 */

(function() {
	// Load plugin specific language pack
	tinymce.PluginManager.requireLangPack('asciimath');

	tinymce.create('tinymce.plugins.AsciimathPlugin', {
		/**
		 * Initializes the plugin, this will be executed after the plugin has been created.
		 * This call is done before the editor instance has finished it's initialization so use the onInit event
		 * of the editor instance to intercept that event.
		 *
		 * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
		 * @param {string} url Absolute URL to where the plugin is located.
		 */
		init : function(ed, url) {
			var t = this;

			// Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand('mceAsciimath');
			ed.addCommand('mceAsciimath', function(val) {

				if (t.lastAMnode==null) {
					existing = ed.selection.getContent();
					if (existing.indexOf('class=AM')==-1) { //existing does not contain an AM node, so turn it into one
					       //strip out all existing html tags.
					       existing = existing.replace(/<([^>]*)>/g,"");
					       existing = existing.replace(/&(m|n)dash;/g,"-");
					       existing = existing.replace(/&?nbsp;?/g," ");
					       existing = existing.replace(/&(.*?);/g,"$1");
					       if (val) {
						       existing = val;
					       }
					       entity = '<span class=AMedit>`'+existing+'<span id="removeme"></span>`</span>&nbsp;';

					       if (tinymce.isIE) ed.focus();

					       ed.selection.setContent(entity);

					       ed.selection.setCursorLocation(ed.dom.get('removeme'),0);
					       ed.dom.remove('removeme');
					       t.justinserted = true;
					       ed.nodeChanged();
					       t.justinserted = null;
					 }

				} else if (val) {
					ed.selection.setContent(' '+val+' ');
				}

			});

			ed.addCommand('mceAsciimathDlg', function() {
				if (typeof AMTcgiloc == 'undefined') {
					AMTcgiloc = "";
				}
				ed.windowManager.open({
					title: "Math Symbols",
					file : url + '/amcharmap.htm',
					width : 720,
					height : 380,
					inline : 1
				}, {
					plugin_url : url, // Plugin absolute URL
					AMTcgiloc : AMTcgiloc
				});

			});
			ed.on('KeyDown', function(ev) {
				if (ev.keyCode == 13 || ev.keyCode == 35 || ev.keyCode == 40) {
				    var rng, AMcontainer, dom = ed.dom;

				    AMcontainer = dom.getParent(ed.selection.getNode(), 'span.AMedit');

				    if (AMcontainer) {
					ed.selection.select(AMcontainer);
					ed.selection.collapse(false);
				    }
				} else if (ev.keyCode == 46 || ev.keyCode == 8) {
					/* handle backspaces - not working :( */
				    node = ed.selection.getNode();
				    var AMcontainer = ed.dom.getParent(node, 'span.AM');
				    if (AMcontainer) {
					AMcontainer.parentNode.removeChild(AMcontainer);
				    }
				}
			});
			ed.on('KeyPress', function (ev) {
				var key = String.fromCharCode(ev.charCode || ev.keyCode);
				if (key=='`') {
					if (t.lastAMnode == null) {
						existing = ed.selection.getContent();
						if (existing.indexOf('class=AM')==-1) { //existing does not contain an AM node, so turn it into one
						       //strip out all existing html tags.
						       existing = existing.replace(/<([^>]*)>/g,"");
						       existing = existing.replace(/&(m|n)dash;/g,"-");
						       existing = existing.replace(/&?nbsp;?/g," ");
						       existing = existing.replace(/&(.*?);/g,"$1");
						       entity = '<span class="AMedit">`'+existing+'<span id="removeme"></span>`</span>&nbsp;';

						       if (tinymce.isIE) ed.focus();

						       ed.selection.setContent(entity);

						       //ed.selection.select(ed.dom.get('removeme'));
						       ed.selection.setCursorLocation(ed.dom.get('removeme'),0);
						       ed.dom.remove('removeme');
						       t.justinserted = true;
						       ed.nodeChanged();
						       t.justinserted = null;
						 }
					}
					if (ev.stopPropagation) {
						ev.stopPropagation();
						ev.preventDefault();
				       } else {
						ev.cancelBubble = true;
						ev.returnValue = false;
				       }
				}
			});

			// Register asciimath button
			ed.addButton('asciimath', {
				tooltip: 'Add New Math',
				cmd : 'mceAsciimath',
				icon: 'mathadd'
				//image : url + '/img/ed_mathformula2.gif'
			});

			ed.addButton('asciimathcharmap', {
				tooltip: 'Math Symbols',
				cmd : 'mceAsciimathDlg',
				icon: 'math'
				//image : url + '/img/ed_mathformula.gif'
			});

			/*ed.on('PreInit', function (e) {
				//took out - was triggering on toolbar clicks
				//tinymce.dom.Event.add(ed.getWin(), 'blur', function(e) {
				//});
				if (tinymce.isIE) {
					addhtml = "<object id=\"mathplayer\" classid=\"clsid:32F66A20-7614-11D4-BD11-00104BD3F987\"></object>";
					addhtml +="<?import namespace=\"m\" implementation=\"#mathplayer\"?>";

					ed.dom.doc.getElementsByTagName("head")[0].insertAdjacentHTML("beforeEnd",addhtml);
				}

			});*/

			//ed.on('PreProcess', function (o) {   //tiny 4 doesn't seem to use this anymore
			//luckily, there is a special paste callback
			ed.on('PastePostProcess', function(o) {
                var AMtags, MJtags;
				//if (o.get) {
                    AMtags = ed.dom.select('span.AM', o.node);
					for (var i=0; i<AMtags.length; i++) {
						t.math2ascii(AMtags[i]);
					}
					MJtags = ed.dom.select('span[data-asciimath]', o.node);
					for (var i=0; i<MJtags.length; i++) {
						t.mathjax2ascii(MJtags[i]);
                    }
                    MJtags = ed.dom.select('mjx-container[data-asciimath]', o.node);
                    for (var i=0; i<MJtags.length; i++) {
                        var newspan = ed.dom.create('span', {'data-asciimath': MJtags[i].getAttribute("data-asciimath")});
                        MJtags[i].parentNode.replaceChild(newspan, MJtags[i]);
						t.mathjax2ascii(newspan);
                    }
					/*AMtags = ed.dom.select('span.AMedit', o.node);
					for (var i=0; i<AMtags.length; i++) {
						var myAM = AMtags[i].innerHTML;
						myAM = "`"+myAM.replace(/\`/g,"")+"`";
						AMtags[i].innerHTML = myAM;
						AMtags[i].className = "AM";
					}*/
				//}


			});

			//converts before source pull
			ed.on('PreProcess', function(o) {
				if (o.get) {
					AMtags = ed.dom.select('span.AM', o.node);
					for (var i=0; i<AMtags.length; i++) {
						t.math2ascii(AMtags[i]);
					}
					AMtags = ed.dom.select('span.AMedit', o.node);
					for (var i=0; i<AMtags.length; i++) {
						var myAM = AMtags[i].innerHTML;
						myAM = "`"+t.cleanmath(myAM)+"`";
						AMtags[i].innerHTML = myAM;
						AMtags[i].className = "AM";
					}
				}
			});

			ed.on('LoadContent', function (o) {
					AMtags = ed.dom.select('span.AM');
					for (var i=0; i<AMtags.length; i++) {
						t.nodeToAM(AMtags[i]);
					}
					t.loaded = true;
			});
			ed.on('SetContent', function (o) {
				if (t.loaded) {
					AMtags = ed.dom.select('span.AM');
					for (var i=0; i<AMtags.length; i++) {
						if (!AMtags[i].innerHTML.match(/(math|img)/)) {
							t.nodeToAM(AMtags[i]);
						}
					}
				}
			});

			ed.on('BeforeSetContent', function (o) {
				o.content = o.content.replace(/(<span[^>]+AM.*?<\/span>)</, "$1 <");

			});
			ed.on('BeforeExecCommand', function (e) {
				if (t.lastAMnode != null) {
					if (e.command == 'Superscript' || e.command == 'Subscript' || e.command == 'Bold' || e.command == 'Subscript' || e.command == 'mceToggleFormat') {
						e.preventDefault();
						return;
					} else if (e.command == 'mceShowCharmap') {
						e.preventDefault();
						ed.execCommand('mceAsciimathDlg');
						return;
					}
				}
				if (e.command != 'mceAsciimath' && e.command != 'mceAsciimathDlg') {
					AMtags = ed.dom.select('span.AM');
					for (var i=0; i<AMtags.length; i++) {
						t.math2ascii(AMtags[i]);
						//AMtags[i].className = "AM";
					}
				}
			});

			ed.on('ExecCommand', function (e) {
				if (e.command != 'mceAsciimath' && e.command != 'mceAsciimathDlg') {
					/*AMtags = ed.dom.select('span.AMedit');
					for (var i=0; i<AMtags.length; i++) {
						t.nodeToAM(AMtags[i]);
						AMtags[i].className = "AM";
					}*/
					AMtags = ed.dom.select('span.AM');
					for (var i=0; i<AMtags.length; i++) {
						if (!AMtags[i].innerHTML.match(/(math|img)/)) {
							AMtags[i].innerHTML = AMtags[i].title;
							t.nodeToAM(AMtags[i]);
						}
					}
					AMimgs = ed.dom.select('img.AMimg');
					for (var i=0; i<AMimgs.length; i++) {
						t.imgwrap(ed, AMimgs[i]);
					}
				}
			});

			//HELP
			ed.on('NodeChange', function (e) {
				var doprocessnode = true;
				if (t.testAMclass(e.element)) {
					p = e.element;
				} else {
					p = ed.dom.getParent(e.element,t.testAMclass);
				}
				//can't figure out how to do in TINY 4
				//cm.setDisabled('charmap', p!=null);
				//cm.setDisabled('sub', p!=null);
				//cm.setDisabled('sup', p!=null);
				if (p != null) {
					if (t.lastAMnode == p) {
						doprocessnode = false;
					} else {
						t.math2ascii(p);
						p.className = 'AMedit';
						if (t.lastAMnode != null) {
							t.nodeToAM(t.lastAMnode);
							t.lastAMnode.className = 'AM';
						}
						if (t.justinserted==null && p.parentNode.innerHTML.match(/<span[^>]*class="AMedit"[^<]*?<\/span>(\s*<br>)?\s*$/)) {
							while (p.parentNode.lastChild.nodeName.toLowerCase()!='span') {
								p.parentNode.removeChild(p.parentNode.lastChild);
							}
							p.parentNode.appendChild(ed.dom.doc.createTextNode("\u00A0"));
						}
						if (t.justinserted==null && p.parentNode.innerHTML.match(/^\s*<span[^>]*class="AMedit"/)) {
							while (p.parentNode.firstChild.nodeName.toLowerCase()!='span') {
								p.parentNode.removeChild(p.parentNode.firstChild);
							}
							p.parentNode.insertBefore(ed.dom.doc.createTextNode("\u00A0"),p.parentNode.firstChild);
						}
						if (t.justinserted==null) {
							ed.selection.setCursorLocation(p,0);
						}
						t.lastAMnode = p;
						doprocessnode = false;
					}
				}
				if (doprocessnode && (t.lastAMnode != null)) { //if not in AM node, process last
				     if (t.lastAMnode.innerHTML.match(/`(&nbsp;|\s|\u00a0|&#160;)*`/) || t.lastAMnode.innerHTML.match(/^(&nbsp;|\s|\u00a0|&#160;)*$/)) {
					     p = t.lastAMnode.parentNode;
					     p.removeChild(t.lastAMnode);
				     } else {
					     t.nodeToAM(t.lastAMnode);
					     t.lastAMnode.className = 'AM';
				     }
				     t.lastAMnode = null;
			       }

			});
			ed.on('Deactivate', function (e) {
				if (t.lastAMnode != null) {
				     if (t.lastAMnode.innerHTML.match(/`(&nbsp;|\s)*`/)|| t.lastAMnode.innerHTML.match(/^(&nbsp;|\s|\u00a0|&#160;)*$/)) {
					     p = t.lastAMnode.parentNode;
					     p.removeChild(t.lastAMnode);
				     } else {
					     t.nodeToAM(t.lastAMnode);
					     t.lastAMnode.className = 'AM';
				     }
				     t.lastAMnode = null;
				}
			});



		},

		/**
		 * Returns information about the plugin as a name/value array.
		 * The current keys are longname, author, authorurl, infourl and version.
		 *
		 * @return {Object} Name/value array containing information about the plugin.
		 */
		getInfo : function() {
			return {
				longname : 'Asciimath plugin',
				author : 'David Lippman',
				authorurl : 'http://dlippman.imathas.com',
				infourl : '',
				version : "4.0"
			};
		},

		math2ascii : function(el) {
			var myAM = el.innerHTML;
			if (myAM.indexOf("`") == -1) {
				if (el.title!='' && el.title.indexOf('math')==-1 && el.title.indexOf('img')==-1) { //myAM.indexOf('math')==-1 && myAM.indexOf('img')==-1 &&
					myAM = el.title;  //if cut-and-paste, grab eqn from title if there
				} else if (myAM.indexOf('title')==-1) {
					myAM = myAM.replace(/.+alt=\"(.*?)\".+/g,"$1");
					myAM = myAM.replace(/.+alt=\'(.*?)\'.+/g,"$1");
					myAM = myAM.replace(/.+alt=([^>]*?)\s.*>.*/g,"$1");
					myAM = myAM.replace(/.+alt=(.*?)>.*/g,"$1");
				} else {
					myAM = myAM.replace(/.+title=\"(.*?)\".+/g,"$1");
					myAM = myAM.replace(/.+title=\'(.*?)\'.+/g,"$1");
					myAM = myAM.replace(/.+title=([^>]*?)\s.*>.*/g,"$1");
					myAM = myAM.replace(/.+title=(.*?)>.*/g,"$1");
				}
				myAM = myAM.replace(/<span[^>]*>(.*?)<\/span>/,"$1");
				myAM = myAM.replace(/<br\s*\/>/," ");
				//myAM = myAM.replace(/&gt;/g,">");
				//myAM = myAM.replace(/&lt;/g,"<");
				myAM = myAM.replace(/>/g,"&gt;");
				myAM = myAM.replace(/</g,"&lt;");
				myAM = "`"+this.cleanmath(myAM)+"`";
				el.innerHTML = myAM;
			}
		},

		mathjax2ascii : function(el) {
			var myAM = el.getAttribute("data-asciimath");
			var attr = el.attributes;
			var i = attr.length;
			while (i--) {
				el.removeAttribute(attr[i].name);
			}
			el.className = "AM";
			el.title = myAM;
			el.innerHTML = "`"+myAM+"`";
		},


		nodeToAM : function(outnode) {
			var str = outnode.innerHTML.replace(/\`/g,"");
            str = str.replace(/<span[^>]*>(.*?)<\/span>/,"$1").replace(/<br\s*\/>/," ")
                .replace(/<img[^>]*title="(.*?)".*?>/g,"$1");
			str = str.replace(/[\u200B-\u200D\uFEFF]/g, '');
			if (tinymce.isIE) {
				  //str.replace(/\"/,"&quot;");
				  var newAM = document.createElement("span");
				  newAM.className = "AM";
				  newAM.appendChild(AMTparseMath(str));

				  outnode.innerHTML = newAM.innerHTML;
				  outnode.title=str; //add title to <span class="AM"> with equation for cut-and-paste
			  } else {
				  //doesn't work on IE, probably because this script is in the parent
				  //windows, and the node is in the iframe.  Should it work in Moz?
				 var myAM = "`"+this.cleanmath(str)+"`"; //next 2 lines needed to make caret
				 outnode.innerHTML = myAM;     //move between `` on Firefox insert math
				 AMprocessNode(outnode);
				 outnode.title=myAM.replace(/\`/g,""); //add title to <span class="AM"> with equation for cut-and-paste
			  }

        },
        
        cleanmath: function(str) {
            return str.replace(/\`/g,"")
                .replace(/<([^>]*)>/g,"")
                .replace(/&(m|n)dash;/g,"-")
                .replace(/&?nbsp;?/g," ")
                .replace(/&(.*?);/g, function(m,p) {
                    if (p=='lt' || p=='gt') {
                        return m;
                    } else {
                        return p;
                    }
                });
        },

		imgwrap : function(ed, imgnode) {
			p = ed.dom.getParent(imgnode,this.testAMclass);
			if (p==null) {
				var newAM = document.createElement("span");
				newAM.className = "AM";
				var rimgnode = imgnode.parentNode.replaceChild(newAM, imgnode);
				newAM.appendChild(rimgnode);
				p = newAM;
			}
		},

		lastAMnode : null,
		loaded : false,
		preventAMrender : false,

		testAMclass : function(el) {
			if ((el.className == 'AM') || (el.className == 'AMedit')) {
				return true;
			} else {
				return false;
			}
		}
	});

	// Register plugin
	tinymce.PluginManager.add('asciimath', tinymce.plugins.AsciimathPlugin);
})();
